@@ -22,9 +22,9 @@
 #include "apr_arch_file_io.h"
 #endif /* APR_HAS_SENDFILE */
 
-/* sys/sysctl.h is only needed on FreeBSD for include_hdrs_in_length() */
-#if defined(__FreeBSD__) && defined(HAVE_SYS_SYSCTL_H)
-#include <sys/sysctl.h>
+/* osreldate.h is only needed on FreeBSD for sendfile detection */
+#if defined(__FreeBSD__)
+#include <osreldate.h>
 #endif
 
 apr_status_t apr_socket_send(apr_socket_t *sock, const char *buf, 
@@ -32,8 +32,8 @@ apr_status_t apr_socket_send(apr_socket_t *sock, const char *buf,
 {
     apr_ssize_t rv;
     
-    if (sock->netmask & APR_INCOMPLETE_WRITE) {
-        sock->netmask &= ~APR_INCOMPLETE_WRITE;
+    if (sock->options & APR_INCOMPLETE_WRITE) {
+        sock->options &= ~APR_INCOMPLETE_WRITE;
         goto do_select;
     }
 
@@ -42,7 +42,7 @@ apr_status_t apr_socket_send(apr_socket_t *sock, const char *buf,
     } while (rv == -1 && errno == EINTR);
 
     while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) 
-        && apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+                    && (sock->timeout > 0)) {
         apr_status_t arv;
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
@@ -60,8 +60,8 @@ do_select:
         *len = 0;
         return errno;
     }
-    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) && rv < *len) {
-        sock->netmask |= APR_INCOMPLETE_WRITE;
+    if ((sock->timeout > 0) && (rv < *len)) {
+        sock->options |= APR_INCOMPLETE_WRITE;
     }
     (*len) = rv;
     return APR_SUCCESS;
@@ -72,8 +72,8 @@ apr_status_t apr_socket_recv(apr_socket_t *sock, char *buf, apr_size_t *len)
     apr_ssize_t rv;
     apr_status_t arv;
 
-    if (sock->netmask & APR_INCOMPLETE_READ) {
-        sock->netmask &= ~APR_INCOMPLETE_READ;
+    if (sock->options & APR_INCOMPLETE_READ) {
+        sock->options &= ~APR_INCOMPLETE_READ;
         goto do_select;
     }
 
@@ -81,8 +81,8 @@ apr_status_t apr_socket_recv(apr_socket_t *sock, char *buf, apr_size_t *len)
         rv = read(sock->socketdes, buf, (*len));
     } while (rv == -1 && errno == EINTR);
 
-    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) && 
-        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+    while ((rv == -1) && (errno == EAGAIN || errno == EWOULDBLOCK)
+                      && (sock->timeout > 0)) {
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 1);
         if (arv != APR_SUCCESS) {
@@ -99,8 +99,8 @@ do_select:
         (*len) = 0;
         return errno;
     }
-    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) && rv < *len) {
-        sock->netmask |= APR_INCOMPLETE_READ;
+    if ((sock->timeout > 0) && (rv < *len)) {
+        sock->options |= APR_INCOMPLETE_READ;
     }
     (*len) = rv;
     if (rv == 0) {
@@ -121,8 +121,8 @@ apr_status_t apr_socket_sendto(apr_socket_t *sock, apr_sockaddr_t *where,
                     where->salen);
     } while (rv == -1 && errno == EINTR);
 
-    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK)
-        && apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+    while ((rv == -1) && (errno == EAGAIN || errno == EWOULDBLOCK)
+                      && (sock->timeout > 0)) {
         apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
         if (arv != APR_SUCCESS) {
             *len = 0;
@@ -154,8 +154,8 @@ apr_status_t apr_socket_recvfrom(apr_sockaddr_t *from, apr_socket_t *sock,
                       (struct sockaddr*)&from->sa, &from->salen);
     } while (rv == -1 && errno == EINTR);
 
-    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) &&
-        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+    while ((rv == -1) && (errno == EAGAIN || errno == EWOULDBLOCK)
+                      && (sock->timeout > 0)) {
         apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 1);
         if (arv != APR_SUCCESS) {
             *len = 0;
@@ -180,10 +180,10 @@ apr_status_t apr_socket_recvfrom(apr_sockaddr_t *from, apr_socket_t *sock,
     return APR_SUCCESS;
 }
 
-#ifdef HAVE_WRITEV
 apr_status_t apr_socket_sendv(apr_socket_t * sock, const struct iovec *vec,
                               apr_int32_t nvec, apr_size_t *len)
 {
+#ifdef HAVE_WRITEV
     apr_ssize_t rv;
     apr_size_t requested_len = 0;
     apr_int32_t i;
@@ -192,8 +192,8 @@ apr_status_t apr_socket_sendv(apr_socket_t * sock, const struct iovec *vec,
         requested_len += vec[i].iov_len;
     }
 
-    if (sock->netmask & APR_INCOMPLETE_WRITE) {
-        sock->netmask &= ~APR_INCOMPLETE_WRITE;
+    if (sock->options & APR_INCOMPLETE_WRITE) {
+        sock->options &= ~APR_INCOMPLETE_WRITE;
         goto do_select;
     }
 
@@ -201,8 +201,8 @@ apr_status_t apr_socket_sendv(apr_socket_t * sock, const struct iovec *vec,
         rv = writev(sock->socketdes, vec, nvec);
     } while (rv == -1 && errno == EINTR);
 
-    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) && 
-        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+    while ((rv == -1) && (errno == EAGAIN || errno == EWOULDBLOCK) 
+                      && (sock->timeout > 0)) {
         apr_status_t arv;
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
@@ -220,14 +220,16 @@ do_select:
         *len = 0;
         return errno;
     }
-    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) &&
-        rv < requested_len) {
-        sock->netmask |= APR_INCOMPLETE_WRITE;
+    if ((sock->timeout > 0) && (rv < requested_len)) {
+        sock->options |= APR_INCOMPLETE_WRITE;
     }
     (*len) = rv;
     return APR_SUCCESS;
-}
+#else
+    *len = vec[0].iov_len;
+    return apr_socket_send(sock, vec[0].iov_base, len);
 #endif
+}
 
 #if APR_HAS_SENDFILE
 
@@ -245,10 +247,36 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
                                  apr_hdtr_t *hdtr, apr_off_t *offset,
                                  apr_size_t *len, apr_int32_t flags)
 {
-    off_t off = *offset;
     int rv, nbytes = 0, total_hdrbytes, i;
     apr_status_t arv;
 
+#if APR_HAS_LARGE_FILES && defined(HAVE_SENDFILE64)
+    apr_off_t off = *offset;
+#define sendfile sendfile64
+
+#elif APR_HAS_LARGE_FILES && SIZEOF_OFF_T == 4
+    /* 64-bit apr_off_t but no sendfile64(): fail if trying to send
+     * past the 2Gb limit. */
+    off_t off;
+    
+    if ((apr_int64_t)*offset + *len > INT_MAX) {
+        return EINVAL;
+    }
+    
+    off = *offset;
+
+#else
+    off_t off = *offset;
+
+    /* Multiple reports have shown sendfile failing with EINVAL if
+     * passed a >=2Gb count value on some 64-bit kernels.  It won't
+     * noticably hurt performance to limit each call to <2Gb at a
+     * time, so avoid that issue here: */
+    if (sizeof(off_t) == 8 && *len > INT_MAX) {
+        *len = INT_MAX;
+    }
+#endif
+
     if (!hdtr) {
         hdtr = &no_hdtr;
     }
@@ -288,8 +316,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         }
     }
 
-    if (sock->netmask & APR_INCOMPLETE_WRITE) {
-        sock->netmask &= ~APR_INCOMPLETE_WRITE;
+    if (sock->options & APR_INCOMPLETE_WRITE) {
+        sock->options &= ~APR_INCOMPLETE_WRITE;
         goto do_select;
     }
 
@@ -300,9 +328,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
                       *len);   /* number of bytes to send */
     } while (rv == -1 && errno == EINTR);
 
-    while (rv == -1 && 
-        (errno == EAGAIN || errno == EWOULDBLOCK) && 
-        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+    while ((rv == -1) && (errno == EAGAIN || errno == EWOULDBLOCK) 
+                      && (sock->timeout > 0)) {
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
         if (arv != APR_SUCCESS) {
@@ -337,8 +364,8 @@ do_select:
              * partial byte count;  this is a non-blocking socket.
              */
 
-            if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
-                sock->netmask |= APR_INCOMPLETE_WRITE;
+            if (sock->timeout > 0) {
+                sock->options |= APR_INCOMPLETE_WRITE;
             }
             return arv;
         }
@@ -372,56 +399,7 @@ do_select:
     return rv < 0 ? errno : APR_SUCCESS;
 }
 
-#elif defined(__FreeBSD__)
-
-static int include_hdrs_in_length(void)
-{
-#ifdef HAVE_SYS_SYSCTL_H
-/* this assumes: 
- *   if the header exists, so does the sysctlbyname() syscall, and 
- *   if the header doesn't exist, the kernel is really old
- */
-
-/* see 
- * http://www.freebsd.org/cgi/cvsweb.cgi/src/sys/sys/param.h#rev1.61.2.29 
- * for kernel version number
- * 
- * http://www.freebsd.org/cgi/cvsweb.cgi/src/sys/kern/uipc_syscalls.c#rev1.65.2.10
- * for the sendfile patch
- */
-#define KERNEL_WITH_SENDFILE_LENGTH_FIX 460001
-
-    typedef enum { UNKNOWN = 0, 
-                   NEW, 
-                   OLD 
-                 } api_e;
-
-    static api_e api; 
-    int kernel_version;  
-    apr_size_t kernel_version_size;
-
-    if (api != UNKNOWN) {
-        return (api == OLD);
-    }
-    kernel_version = 0;    /* silence compiler warning */
-    kernel_version_size = sizeof(kernel_version);
-    if (sysctlbyname("kern.osreldate", &kernel_version, 
-                     &kernel_version_size, NULL, 0) == 0 &&
-        kernel_version < KERNEL_WITH_SENDFILE_LENGTH_FIX) {
-        api = OLD;
-        return 1;
-    }
-    /* size of kern.osreldate's output might change in the future 
-     * causing the sysctlbyname to fail,
-     * but if it's the future, we should use the newer API 
-     */
-    api = NEW;
-    return 0;
-#else
-    /* the build system's kernel is older than 3.4.  Use the old API */
-    return 1;
-#endif
-}
+#elif defined(__FreeBSD__) || defined(__DragonFly__)
 
 /* Release 3.1 or greater */
 apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
@@ -440,7 +418,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
         hdtr = &no_hdtr;
     }
 
-    else if (hdtr->numheaders && include_hdrs_in_length()) {
+#if defined(__FreeBSD_version) && __FreeBSD_version < 460001
+    else if (hdtr->numheaders) {
 
         /* On early versions of FreeBSD sendfile, the number of bytes to send 
          * must include the length of the headers.  Don't look at the man page 
@@ -453,6 +432,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
             bytes_to_send += hdtr->headers[i].iov_len;
         }
     }
+#endif
 
     headerstruct.headers = hdtr->headers;
     headerstruct.hdr_cnt = hdtr->numheaders;
@@ -461,9 +441,9 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
 
     /* FreeBSD can send the headers/footers as part of the system call */
     do {
-        if (sock->netmask & APR_INCOMPLETE_WRITE) {
+        if (sock->options & APR_INCOMPLETE_WRITE) {
             apr_status_t arv;
-            sock->netmask &= ~APR_INCOMPLETE_WRITE;
+            sock->options &= ~APR_INCOMPLETE_WRITE;
             arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
             if (arv != APR_SUCCESS) {
                 *len = 0;
@@ -485,8 +465,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
 
             if (rv == -1) {
                 if (errno == EAGAIN) {
-                    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
-                        sock->netmask |= APR_INCOMPLETE_WRITE;
+                    if (sock->timeout > 0) {
+                        sock->options |= APR_INCOMPLETE_WRITE;
                     }
                     /* FreeBSD's sendfile can return -1/EAGAIN even if it
                      * sent bytes.  Sanitize the result so we get normal EAGAIN
@@ -523,9 +503,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
                 nbytes = 0;
             }
         }
-        if (rv == -1 &&
-            errno == EAGAIN && 
-            apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+        if ((rv == -1) && (errno == EAGAIN) 
+                       && (sock->timeout > 0)) {
             apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
             if (arv != APR_SUCCESS) {
                 *len = 0;
@@ -566,6 +545,24 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
     struct iovec hdtrarray[2];
     char *headerbuf, *trailerbuf;
 
+#if APR_HAS_LARGE_FILES && defined(HAVE_SENDFILE64)
+    /* later HP-UXes have a sendfile64() */
+#define sendfile sendfile64
+    apr_off_t off = *offset;
+
+#elif APR_HAS_LARGE_FILES && SIZEOF_OFF_T == 4
+    /* HP-UX 11.00 doesn't have a sendfile64(): fail if trying to send
+     * past the 2Gb limit */
+    off_t off;
+
+    if ((apr_int64_t)*offset + *len > INT_MAX) {
+        return EINVAL;
+    }
+    off = *offset;
+#else
+    apr_off_t off = *offset;
+#endif
+
     if (!hdtr) {
         hdtr = &no_hdtr;
     }
@@ -592,7 +589,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         }  
 
         /* XXX:  BUHHH? wow, what a memory leak! */
-        headerbuf = hdtrarray[0].iov_base = apr_palloc(sock->cntxt, headerlen);
+        headerbuf = hdtrarray[0].iov_base = apr_palloc(sock->pool, headerlen);
         hdtrarray[0].iov_len = headerlen;
 
         for (i = 0; i < hdtr->numheaders; i++) {
@@ -617,7 +614,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         }
 
         /* XXX:  BUHHH? wow, what a memory leak! */
-        trailerbuf = hdtrarray[1].iov_base = apr_palloc(sock->cntxt, trailerlen);
+        trailerbuf = hdtrarray[1].iov_base = apr_palloc(sock->pool, trailerlen);
         hdtrarray[1].iov_len = trailerlen;
 
         for (i = 0; i < hdtr->numtrailers; i++) {
@@ -631,7 +628,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         if (nbytes) {       /* any bytes to send from the file? */
             rc = sendfile(sock->socketdes,      /* socket  */
                           file->filedes,        /* file descriptor to send */
-                          *offset,              /* where in the file to start */
+                          off,                  /* where in the file to start */
                           nbytes,               /* number of bytes to send from file */
                           hdtrarray,            /* Headers/footers */
                           flags);               /* undefined, set to 0 */
@@ -641,9 +638,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         }
     } while (rc == -1 && errno == EINTR);
 
-    while (rc == -1 && 
-        (errno == EAGAIN || errno == EWOULDBLOCK) && 
-        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+    while ((rc == -1) && (errno == EAGAIN || errno == EWOULDBLOCK) 
+                      && (sock->timeout > 0)) {
         apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
 
         if (arv != APR_SUCCESS) {
@@ -655,7 +651,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
                 if (nbytes) {
                     rc = sendfile(sock->socketdes,    /* socket  */
                                   file->filedes,      /* file descriptor to send */
-                                  *offset,            /* where in the file to start */
+                                  off,                /* where in the file to start */
                                   nbytes,             /* number of bytes to send from file */
                                   hdtrarray,          /* Headers/footers */
                                   flags);             /* undefined, set to 0 */
@@ -729,7 +725,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
             /* but headers are small, so maybe we can hold on to the
              * memory for the life of the socket...
              */
-            hbuf = apr_palloc(sock->cntxt, parms.header_length);
+            hbuf = apr_palloc(sock->pool, parms.header_length);
 #endif
             ptr = 0;
             for (i = 0; i < hdtr->numheaders; i++) {
@@ -755,7 +751,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
             /* Keepalives make apr_palloc a bad idea */
             tbuf = malloc(parms.trailer_length);
 #else
-            tbuf = apr_palloc(sock->cntxt, parms.trailer_length);
+            tbuf = apr_palloc(sock->pool, parms.trailer_length);
 #endif
             ptr = 0;
             for (i = 0; i < hdtr->numtrailers; i++) {
@@ -778,8 +774,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
 
     /* O.K. All set up now. Let's go to town */
 
-    if (sock->netmask & APR_INCOMPLETE_WRITE) {
-        sock->netmask &= ~APR_INCOMPLETE_WRITE;
+    if (sock->options & APR_INCOMPLETE_WRITE) {
+        sock->options &= ~APR_INCOMPLETE_WRITE;
         goto do_select;
     }
 
@@ -789,9 +785,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
                        flags);             /* flags */
     } while (rv == -1 && errno == EINTR);
 
-    while (rv == -1 &&
-        (errno == EAGAIN || errno == EWOULDBLOCK) &&
-        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
+    while ((rv == -1) && (errno == EAGAIN || errno == EWOULDBLOCK) 
+                      && (sock->timeout > 0)) {
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
         if (arv != APR_SUCCESS) {
@@ -819,9 +814,10 @@ do_select:
         return errno;
     }
 
-    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) &&
-        (parms.bytes_sent < (parms.file_bytes + parms.header_length + parms.trailer_length))) {
-        sock->netmask |= APR_INCOMPLETE_WRITE;
+    if ((sock->timeout > 0)
+          && (parms.bytes_sent 
+                < (parms.file_bytes + parms.header_length + parms.trailer_length))) {
+        sock->options |= APR_INCOMPLETE_WRITE;
     }
 
     return APR_SUCCESS;
@@ -843,6 +839,12 @@ do_select:
  * 111298-01, 108529-09, 109473-06, 109235-04, 108996-02, 111296-01, 109026-04,
  * 108992-13
  */
+
+#if APR_HAS_LARGE_FILES && defined(HAVE_SENDFILEV64)
+#define sendfilevec_t sendfilevec64_t
+#define sendfilev sendfilev64
+#endif
+
 apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
                                  apr_hdtr_t *hdtr, apr_off_t *offset,
                                  apr_size_t *len, apr_int32_t flags)
@@ -862,7 +864,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
 
     /* Calculate how much space we need. */
     vecs = hdtr->numheaders + hdtr->numtrailers + 1;
-    sfv = apr_palloc(sock->cntxt, sizeof(sendfilevec_t) * vecs);
+    sfv = apr_palloc(sock->pool, sizeof(sendfilevec_t) * vecs);
 
     curvec = 0;
 
@@ -870,7 +872,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
     for (i = 0; i < hdtr->numheaders; i++, curvec++) {
         sfv[curvec].sfv_fd = SFV_FD_SELF;
         sfv[curvec].sfv_flag = 0;
-        sfv[curvec].sfv_off = (off_t)hdtr->headers[i].iov_base;
+        sfv[curvec].sfv_off = (apr_off_t)hdtr->headers[i].iov_base;
         sfv[curvec].sfv_len = hdtr->headers[i].iov_len;
         requested_len += sfv[curvec].sfv_len;
     }
@@ -894,7 +896,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
     for (i = 0; i < hdtr->numtrailers; i++, curvec++) {
         sfv[curvec].sfv_fd = SFV_FD_SELF;
         sfv[curvec].sfv_flag = 0;
-        sfv[curvec].sfv_off = (off_t)hdtr->trailers[i].iov_base;
+        sfv[curvec].sfv_off = (apr_off_t)hdtr->trailers[i].iov_base;
         sfv[curvec].sfv_len = hdtr->trailers[i].iov_len;
         requested_len += sfv[curvec].sfv_len;
     }
@@ -902,8 +904,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
     /* If the last write couldn't send all the requested data,
      * wait for the socket to become writable before proceeding
      */
-    if (sock->netmask & APR_INCOMPLETE_WRITE) {
-        sock->netmask &= ~APR_INCOMPLETE_WRITE;
+    if (sock->options & APR_INCOMPLETE_WRITE) {
+        sock->options &= ~APR_INCOMPLETE_WRITE;
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
         if (arv != APR_SUCCESS) {
             *len = 0;
@@ -933,8 +935,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
             if (nbytes) {
                 rv = 0;
             }
-            else if (!arv &&
-                     apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) == 1) {
+            else if (!arv && (sock->timeout > 0)) {
                 apr_status_t t = apr_wait_for_io_or_timeout(NULL, sock, 0);
 
                 if (t != APR_SUCCESS) {
@@ -955,60 +956,16 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
 
     /* Update how much we sent */
     *len = nbytes;
-    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) &&
-        (*len < requested_len)) {
-        sock->netmask |= APR_INCOMPLETE_WRITE;
+    if ((sock->timeout > 0) && (*len < requested_len)) {
+        sock->options |= APR_INCOMPLETE_WRITE;
     }
     return APR_SUCCESS;
 }
 #else
 #error APR has detected sendfile on your system, but nobody has written a
-#error version of it for APR yet.  To get past this, either write apr_sendfile
-#error or change APR_HAS_SENDFILE in apr.h to 0. 
-#endif /* __linux__, __FreeBSD__, __HPUX__, _AIX, __MVS__, Tru64/OSF1 */
-
-/* deprecated */
-apr_status_t apr_sendfile(apr_socket_t *sock, apr_file_t *file,
-                          apr_hdtr_t *hdtr, apr_off_t *offset, apr_size_t *len,
-                          apr_int32_t flags)
-{
-    return apr_socket_sendfile(sock, file, hdtr, offset, len, flags);
-}
+#error version of it for APR yet.  To get past this, either write 
+#error apr_socket_sendfile or change APR_HAS_SENDFILE in apr.h to 0.
+#endif /* __linux__, __FreeBSD__, __DragonFly__, __HPUX__, _AIX, __MVS__,
+	  Tru64/OSF1 */
 
 #endif /* APR_HAS_SENDFILE */
-
-/* deprecated */
-apr_status_t apr_send(apr_socket_t *sock, const char *buf, apr_size_t *len)
-{
-    return apr_socket_send(sock, buf, len);
-}
-
-/* deprecated */
-#ifdef HAVE_WRITEV
-apr_status_t apr_sendv(apr_socket_t * sock, const struct iovec *vec,
-                       apr_int32_t nvec, apr_size_t *len)
-{
-    return apr_socket_sendv(sock, vec, nvec, len);
-}
-#endif
-
-/* deprecated */
-apr_status_t apr_sendto(apr_socket_t *sock, apr_sockaddr_t *where,
-                        apr_int32_t flags, const char *buf, apr_size_t *len)
-{
-    return apr_socket_sendto(sock, where, flags, buf, len);
-}
-
-/* deprecated */
-apr_status_t apr_recvfrom(apr_sockaddr_t *from, apr_socket_t *sock,
-                          apr_int32_t flags, char *buf, 
-                          apr_size_t *len)
-{
-    return apr_socket_recvfrom(from, sock, flags, buf, len);
-}
-
-/* deprecated */
-apr_status_t apr_recv(apr_socket_t *sock, char *buf, apr_size_t *len)
-{
-    return apr_socket_recv(sock, buf, len);
-}
